// Shadow.c
// OpenGL SuperBible
// Demonstrates simple planar shadows
// Program by Richard S. Wright Jr.

#include "../Common/OpenGLSB.h"  // System and OpenGL Stuff
#include "../Common/gltools.h"       // gltools library

		int k;

struct CPoint{
	float x,y,z;
};
	const float SX=150.0f;
	const float SZ=200.0f;
	const float SY=200.0f;



float GRAPH_QUALITY = 40;

#define SCENE_COLOR 0
#define PYRAMID_COLOR 1
#define SPHERE_COLOR 2
#define LIGHT0_COLOR 3
#define LIGHT1_COLOR 4
#define CLEAR_COLOR 5
#define MAX_COLORS 6

GLint Colors[MAX_COLORS][4];
GLboolean IsOrtho = false;
GLboolean IsSolid = true;
GLboolean Spot    = false;
GLfloat dx = 0.0f, dy = 0.0f, dz = -5.0f, ddz = 0.1f;
GLfloat alpha = 0.0f, theta = 0.0f;
GLfloat SceneAngleY=0.0f,SceneAngleX=0.0f;

	GLfloat ambient[]={0.1f,0.1f,0.1f,1.0f};
	GLfloat color[]={0.0f,0.5f,0.0f,1.0f};
	GLfloat green[]={0.0f,1.0f,0.0f,1.0f};
	GLfloat white[]={1.0f,1.0f,1.0f,1.0f};


GLfloat pos[]={0.0f,20.0f,0.0f,1.0f};
GLfloat pos1[]={-100.0f,-20.0f,0.0f,1.0f};


// Rotation amounts
static GLfloat xRot = 0.0f;
static GLfloat yRot = 0.0f;

// These values need to be available globally
// Light values and coordinates
GLfloat  ambientLight[] = { 0.0f, 0.0f, 0.0f, 1.0f };
GLfloat  diffuseLight[] = { 0.7f, 0.7f, 0.7f, 1.0f };
GLfloat  specular[] = { 0.3f, 0.3f, 0.3f, 1.0f};
GLfloat  lightPos[] = { 0.0f, 100.0f, 100.0f, 0.0f };
GLfloat  specref[] =  { 1.0f, 1.0f, 1.0f, 1.0f };


GLfloat  ambientLight2[] = { 0.3f, 0.3f, 0.6f, 1.0f };
GLfloat  diffuseLight2[] = { 0.3f, 0.3f, 0.6f, 1.0f };
GLfloat  specular2[] = { 0.3f, 0.3f, 0.6f, 1.0f};




// Transformation matrix to project shadow
GLTMatrix shadowMat;
GLTMatrix shadowMat2;


void PlaneParam(int p1,int p2, int p3, GLdouble * coeffic, CPoint * kub)
{
	coeffic[0] = kub[p1].y * (kub[p2].z - kub[p3].z) + kub[p2].y * (kub[p3].z - kub[p1].z) + kub[p3].y * (kub[p1].z - kub[p2].z);
	coeffic[1] = kub[p1].z * (kub[p2].x - kub[p3].x) + kub[p2].z * (kub[p3].x - kub[p1].x) + kub[p3].z * (kub[p1].x - kub[p2].x);
	coeffic[2] = kub[p1].x * (kub[p2].y - kub[p3].y) + kub[p2].x * (kub[p3].y - kub[p1].y) + kub[p3].x * (kub[p1].y - kub[p2].y);
	coeffic[3] = -kub[p1].x * (kub[p2].y*kub[p3].z - kub[p3].y*kub[p2].z)
				 -kub[p2].x * (kub[p3].y*kub[p1].z - kub[p1].y*kub[p3].z) 
				 -kub[p3].x * (kub[p1].y*kub[p2].z - kub[p2].y*kub[p1].z); 				
}


////////////////////////////////////////////////
// This function just specifically draws the jet
void DrawJet(int nShadow)
  {


  GLTVector3 vNormal; // Storeage for calculated surface normal

  // Nose Cone /////////////////////////////
  // Set material color, note we only have to set to black
  // for the shadow once
  if(nShadow == 0)
           glColor3ub(150, 0,k);
  else
            glColor3ub(1,1,1);

	float Shininess=127;


	glMatrixMode(GL_MODELVIEW);
	

	glPushMatrix();
	const float G_SIZE=7.0f;
	glScalef(G_SIZE,G_SIZE,G_SIZE);
	//if (!IsOrtho) 
	//	glTranslatef(0.0f, 0.0f, -200.0f);
	//glRotatef(theta, 0.0f, 0.0f, 1.0f);
		

	//	gluLookAt(0.0f,50.0f,200.0f,
	//		0.0f,0.0f,0.0f,
	//		0.0f,1.0f,0.0f);
	glTranslatef(dx,dy,dz);

	
	//glRotatef(SceneAngleX,0.0f,0.0f,-1.0f*sinf(SceneAngleY/360*3.1415926545));
	static const float cutoff=25;
	static const float cutoff180=180;
	static const float exponent=10.0f;
	
	

	glRotatef(SceneAngleY,0.0f,1.0f,0.0f);
	glRotatef(SceneAngleX,1.0f,0.0f,1.0f);	

	glPushMatrix();
	glRotatef(alpha,0.0f,1.0f,0.0f);
	//glTranslatef(-5.0f,0.0f,0.0f);
	
	//glMaterialiv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,Colors[SPHERE_COLOR]);
	//glMaterialiv(GL_FRONT,GL_SPECULAR,Colors[SPHERE_COLOR]);
	//glMaterialfv(GL_FRONT,GL_SHININESS,&Shininess);
	if (IsSolid) glutSolidSphere(2.0f,GRAPH_QUALITY,GRAPH_QUALITY);
		else glutWireSphere(2.0f,GRAPH_QUALITY,GRAPH_QUALITY);
	glPopMatrix();

	
	  if(nShadow == 0)
           glColor3ub(0, 70,k);
  else
            glColor3ub(1,1,1);

	glPushMatrix();
	glRotatef(theta,0.0f,1.0f,0.0f);

	glTranslatef(-5.0f,0.0f,0.0f);
	
	//glMaterialiv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,Colors[PYRAMID_COLOR]);
	//glMaterialiv(GL_FRONT,GL_SPECULAR,Colors[PYRAMID_COLOR]);
	
	glBegin(GL_TRIANGLE_FAN);
	
		glNormal3f(0.0f,-1.0f,0.0f);
		glVertex3f(-3.0f,-3.0f,-3.0f);
		glVertex3f(3.0f,-3.0f,-3.0f);
		glVertex3f(3.0f,-3.0f,3.0f);
		glVertex3f(-3.0f,-3.0f,3.0f);
	glEnd();
	glBegin(GL_TRIANGLE_FAN);
		glNormal3f(0.0f,1.0f,0.0f);
		glVertex3f(-2.0f,3.0f,-2.0f);
		glVertex3f(-2.0f,3.0f,2.0f);
		glVertex3f(2.0f,3.0f,2.0f);
		glVertex3f(2.0f,3.0f,-2.0f);
	glEnd();
	glBegin(GL_TRIANGLE_FAN);
		glNormal3f(0.0f,1.0f,6.0f);
		glVertex3f(-2.0f,3.0f,2.0f);
		glVertex3f(-3.0f,-3.0f,3.0f);
		glVertex3f(3.0f,-3.0f,3.0f);
		glVertex3f(2.0f,3.0f,2.0f);
	glEnd();
	glBegin(GL_TRIANGLE_FAN);
		glNormal3f(0.0f,1.0f,-6.0f);
		glVertex3f(-2.0f,3.0f,-2.0f);
		glVertex3f(2.0f,3.0f,-2.0f);
		glVertex3f(3.0f,-3.0f,-3.0f);
		glVertex3f(-3.0f,-3.0f,-3.0f);
	glEnd();
	
	glBegin(GL_TRIANGLE_FAN);
		glNormal3f(-6.0f,1.0f,0.0f);
		glVertex3f(-2.0f,3.0f,-2.0f);
		glVertex3f(-3.0f,-3.0f,-3.0f);
		glVertex3f(-3.0f,-3.0f,3.0f);
		glVertex3f(-2.0f,3.0f,2.0f);
	glEnd();
	glBegin(GL_QUADS);
		glNormal3f(6.0f,1.0f,0.0f);
		glVertex3f(2.0f,3.0f,-2.0f);
		glVertex3f(2.0f,3.0f,2.0f);
		glVertex3f(3.0f,-3.0f,3.0f);
		glVertex3f(3.0f,-3.0f,-3.0f);
	glEnd();
	glPopMatrix();
	
	
	



/*

	glEnable(GL_LIGHT0);
	
	glLightfv(GL_LIGHT0,GL_POSITION,pos);
	static GLfloat dir[]={0.0f,-1.0f,0.0f};
	glLightfv(GL_LIGHT0,GL_SPOT_DIRECTION,dir);
	glLightiv(GL_LIGHT0,GL_DIFFUSE,Colors[LIGHT0_COLOR]);
	glLightiv(GL_LIGHT0,GL_SPECULAR,Colors[LIGHT0_COLOR]);

	if (Spot)
	{
		glLightfv(GL_LIGHT0,GL_SPOT_CUTOFF,&cutoff);
		glLightfv(GL_LIGHT0,GL_SPOT_EXPONENT,&exponent);
	}
	else
	{
		glLightfv(GL_LIGHT0,GL_SPOT_CUTOFF,&cutoff180);
	}


	glEnable(GL_LIGHT1);
	static GLfloat dir1[]={1.0f,0.0f,0.0f};

	glLightfv(GL_LIGHT1,GL_POSITION,pos1);
	glLightfv(GL_LIGHT1,GL_SPOT_DIRECTION,dir1);
	glLightiv(GL_LIGHT1,GL_DIFFUSE,Colors[LIGHT1_COLOR]);
	glLightiv(GL_LIGHT1,GL_SPECULAR,Colors[LIGHT1_COLOR]);
	
	if (Spot)
	{
		glLightfv(GL_LIGHT1,GL_SPOT_CUTOFF,&cutoff);
		glLightfv(GL_LIGHT0,GL_SPOT_EXPONENT,&exponent);
	}
	else
	{
		glLightfv(GL_LIGHT1,GL_SPOT_CUTOFF,&cutoff180);
	}

/*	glTranslatef(-dx,-dy,-dz);

	
	glMaterialiv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,&(Colors[SCENE_COLOR][0]));
	glMaterialiv(GL_FRONT,GL_SPECULAR,&(Colors[SCENE_COLOR][0]));
	const float SIZE=100.0f;
	const float SIZER=100.0f/float(GRAPH_QUALITY)+1.0f;

	glNormal3f(0.0f,1.0f,0.0f);
	for (float n=-SIZE;n<SIZE;n+=SIZER)
	{
		for (float m=-SIZE; m<SIZE; m+=SIZER)
		{
			glBegin(GL_POLYGON);
				glVertex3i(n,-30,m);
				glVertex3i(n,-30,m+SIZER);
				glVertex3i(n+SIZER,-30,m+SIZER);
				glVertex3i(n+SIZER,-30,m);
				
			glEnd();
		}
	}

*/
	

	glPopMatrix();

}

// Called to draw scene
void RenderScene(void)
    {

		float xRot2=xRot,yRot2=yRot;
    // Clear the window with current clearing color
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	int Situation=0;
	while (Situation<=2)
	{
	glPushMatrix();
	
	
	    // Draw the ground, we do manual shading to a darker green
    // in the background to give the illusion of depth
    // Now rotate the jet around in the new flattend space
    glRotatef(xRot2, 1.0f, 0.0f, 0.0f);
    glRotatef(yRot2, 0.0f, 1.0f, 0.0f);
	glDisable(GL_CLIP_PLANE0);
	glDisable(GL_CLIP_PLANE1);
	glDisable(GL_CLIP_PLANE2);
	glDisable(GL_CLIP_PLANE3);
	glDisable(GL_CLIP_PLANE4);

		switch (Situation)
		{
		case 0:

			glCullFace(GL_BACK);
			glDisable(GL_DEPTH_TEST);
			glBegin(GL_QUADS);
				glColor4f(0.5f,0.5f,1.0f,0.9f);
				glNormal3f(1.0f,0.0f,0.0f);
				glVertex3f(-SX,SY,SZ);
				glVertex3f(-SX,-SY,SZ);
				glVertex3f(-SX,-SY,-SZ);
				glVertex3f(-SX,SY,-SZ);
	

				//glNormal3f(-1.0f,0.0f,0.0f);
				//glVertex3f(-SX,SY,-SZ);
				//glVertex3f(-SX,-SY,-SZ);
				//glVertex3f(-SX,-SY,SZ);
				//glVertex3f(-SX,SY,SZ);
				
				
				
			glEnd();
			
			glScalef(-1.0f,1.0f,1.0f);
			glTranslatef(SX*2,0.0f,0.0f);
			
			glCullFace(GL_FRONT);
			k=130;
			break;
		case 1:

			//glScalef(-1.0f,1.0f,1.0f);
			//glTranslatef(-SX*2,0.0f,0.0f);
			//k=130;
		  
			//break;
		case 2:
			//glScalef(1.0f,0.0f,0.0f);
			k=0;
			glCullFace(GL_BACK);
			break;
			
		}



	glEnable(GL_LIGHTING);
	
	


	






	const float SM=0.1f;
	const float SX2=SX+SM,SY2=SY+SM,SZ2=SZ+SM;
	CPoint kub[8] = {{-SX2,-SY2,-SZ2},{-SX2,-SY2, SZ2},{SX2,-SY2,SZ2},{SX2,-SY2,-SZ2,},{-SX2,SY2,-SZ2},{-SX2,SY2, SZ2},{SX2,SY2,SZ2},{SX2,SY2,-SZ2,}};						    //  
	/*CPoint kub2[8] = {{-1,-1,-1},{-1,-1, 1},{1,-1,1},{40,-40,-40,},{-40,40,-40},{-40,40, 40},{40,40,40},{40,40,-40,}};	
	glClearColor(0.0,0.0,0.0,1.0);			//  
	glPolygonMode(GL_FRONT_AND_BACK, Gradient);
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_NORMALIZE);
	glEnable(GL_DEPTH_TEST);
	
	GLfloat atmoColor[4] = {0.8, 0.8, 1.0, 0.005};
	glFogfv (GL_FOG_COLOR, atmoColor);
	//glFogf (GL_FOG_START, 0.5);
	//glFogf (GL_FOG_END, 0.5);
	glFogf (GL_FOG_DENSITY, 0.005);
	glEnable(GL_FOG);

//---------------------------------------------------------------------------//
	GLTVector3 points_w1[3] = {{-40,40,40},{-40,-40,40},{-40,40, -40}};
	GLTVector3 points_w2[3] = {{40,40,-40},{-40,40, -40},{40,-40,-40}};
	GLTVector3 points_w3[3] = {{40,40,40},{40,40,-40},{40,-40,40}};
	GLTVector3 points_f[3] = {{-40,-40,-40},{-40,-40, 40},{40,-40,40}};
	
	GLfloat lightPos_f[] = {0.0, 38.0,0.0,1.0};
//---------------------------------------------------------------------------//*/
	GLdouble coeffic_f[4] = {0.0,0.0,0.0,0.0};
	GLdouble coeffic_w1[4] = {0.0,0.0,0.0,0.0};
	GLdouble coeffic_w2[4] = {0.0,0.0,0.0,0.0};
	GLdouble coeffic_w3[4] = {0.0,0.0,0.0,0.0};
	GLdouble coeffic_w4[4] = {0.0,0.0,0.0,0.0};
		
	PlaneParam(0,1,2,coeffic_f,kub);
	PlaneParam(4,5,1,coeffic_w1,kub);
	PlaneParam(7,4,0,coeffic_w2,kub);
	PlaneParam(2,6,7,coeffic_w3,kub);
	PlaneParam(1,5,6,coeffic_w4,kub);
	
	glClipPlane(GL_CLIP_PLANE0,coeffic_f);
	glClipPlane(GL_CLIP_PLANE1,coeffic_w1);
	glClipPlane(GL_CLIP_PLANE2,coeffic_w2);
	glClipPlane(GL_CLIP_PLANE3,coeffic_w3);
	glClipPlane(GL_CLIP_PLANE4,coeffic_w4);
	glEnable(GL_CLIP_PLANE0);
	glEnable(GL_CLIP_PLANE1);
	glEnable(GL_CLIP_PLANE2);
	glEnable(GL_CLIP_PLANE3);
	glEnable(GL_CLIP_PLANE4);





	glEnable(GL_DEPTH_TEST);
      glBegin(GL_QUADS);

        glColor3ub(0,255,k);
		glNormal3f(0.0f,1.0f,0.0f);
        glVertex3f(SX, -SY, -SZ);
        glVertex3f(-SX, -SY, -SZ);
        glColor3ub(0,255,k);
        glVertex3f(-SX, -SY, SZ);
        glVertex3f(SX, -SY, SZ);



		glNormal3f(0.0f,-1.0f,0.0f);

		glColor3ub(0,255,k);
		glVertex3f(SX, SY, SZ);
		glVertex3f(-SX, SY, SZ);
		 glColor3ub(0,255,k);
		
		glVertex3f(-SX, SY, -SZ);
        glVertex3f(SX, SY, -SZ);
        
        
        
        

		//////////////////////////////
		//glColor3ub(k,132,k);
		glNormal3f(0.0f,0.0f,1.0f);
		glVertex3f(-SX,SY,-SZ);
		glVertex3f(-SX,-SY,-SZ);
		glVertex3f(SX,-SY,-SZ);
		glVertex3f(SX,SY,-SZ);

	

    glEnd();


	    // Save the matrix state and do the rotations
    glPushMatrix();
    // Draw jet at new orientation, put light in correct position
    // before rotating the jet
    glEnable(GL_LIGHTING);
	glEnable(GL_DEPTH_TEST);
    glLightfv(GL_LIGHT0,GL_POSITION,lightPos);
    //glRotatef(xRot2, 1.0f, 0.0f, 0.0f);
    //glRotatef(yRot2, 0.0f, 1.0f, 0.0f);

    DrawJet(0);

    // Restore original matrix state
    glPopMatrix();  


		


    // Get ready to draw the shadow and the ground
    // First disable lighting and save the projection state
   // glDisable(GL_DEPTH_TEST);
    glDisable(GL_LIGHTING);
    // Get ready to draw the shadow and the ground
    // First disable lighting and save the projection state
  
    glPushMatrix();

    // Multiply by shadow projection matrix
    glMultMatrixf((GLfloat *)shadowMat);

    // Now rotate the jet around in the new flattend space
    //glRotatef(xRot2, 1.0f, 0.0f, 0.0f);
   // glRotatef(yRot2, 0.0f, 1.0f, 0.0f);

    // Pass true to indicate drawing shadow
    DrawJet(1); 

    // Restore the projection to normal
    glPopMatrix();


	


		


    glPushMatrix();

    // Multiply by shadow projection matrix
    glMultMatrixf((GLfloat *)shadowMat2);

    // Now rotate the jet around in the new flattend space
    //glRotatef(xRot2, 1.0f, 0.0f, 0.0f);
    //glRotatef(yRot2, 0.0f, 1.0f, 0.0f);

    // Pass true to indicate drawing shadow
    DrawJet(1); 

    // Restore the projection to normal
    glPopMatrix();




	glDisable(GL_LIGHTING);
    // Draw the light source
    glPushMatrix();
    glTranslatef(lightPos[0],lightPos[1], lightPos[2]);
    glColor3ub(255,255,0);
    glutSolidSphere(5.0f,10,10);
    glPopMatrix();
	glEnable(GL_LIGHTING);
    // Restore lighting state variables
    glEnable(GL_DEPTH_TEST);

	glPopMatrix();
    // Display the results

  
	Situation++;
	}
	  glutSwapBuffers();
}

// This function does any needed initialization on the rendering
// context. 
void SetupRC()
    {
    // Any three points on the ground (counter clockwise order)
	const float SY2=SY-1.0f;
	const float SZ2=SZ-1.0f;
    GLTVector3 points[3] = {{ -30.0f, -SY2, -20.0f },
                            { -30.0f, -SY2, 20.0f },
                            { 40.0f, -SY2, 20.0f }};
	GLTVector3 points2[3] = {{ -30.0f, SY2, -SZ2 },
                            { -30.0f, -SY2, -SZ2 },
                            { 40.0f, -SY2, -SZ2 }};

    glEnable(GL_DEPTH_TEST);  // Hidden surface removal
    glFrontFace(GL_CCW);    // Counter clock-wise polygons face out
    glEnable(GL_CULL_FACE);   // Do not calculate inside of jet

    // Setup and enable light 0
    glLightfv(GL_LIGHT0,GL_AMBIENT,ambientLight);
    glLightfv(GL_LIGHT0,GL_DIFFUSE,diffuseLight);
    glLightfv(GL_LIGHT0,GL_SPECULAR,specular);
    glLightfv(GL_LIGHT0,GL_POSITION,lightPos);
    glEnable(GL_LIGHT0);

    // Enable color tracking
    glEnable(GL_COLOR_MATERIAL);
  
    // Set Material properties to follow glColor values
    glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);

    // All materials hereafter have full specular reflectivity
    // with a high shine
    glMaterialfv(GL_FRONT, GL_SPECULAR,specref);
    glMateriali(GL_FRONT,GL_SHININESS,128);

    // Light blue background
    glClearColor(0.0f, 0.0f, 0.0f, 1.0f );

    // Calculate projection matrix to draw shadow on the ground
    gltMakeShadowMatrix(points, lightPos, shadowMat);
	gltMakeShadowMatrix(points2, lightPos, shadowMat2);

		for (int n=0; n<MAX_COLORS;n++)
		{
			
		//	Colors[n][1]=0x3fffffff;
			Colors[n][2]=0x7fffffff;
		//	Colors[n][3]=0x7fffffff;
		}


	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_NORMALIZE);
	glEnable(GL_FOG);
	float col[]={1.0f,1.0f,1.0f,1.0f};
	glFogfv(GL_FOG_COLOR,col);
	float den=0.0009;
	glFogf(GL_FOG_DENSITY,den);

    }

void SpecialKeys(int key, int x, int y)
    {
    if(key == GLUT_KEY_UP)
        xRot-= 5.0f;

    if(key == GLUT_KEY_DOWN)
        xRot += 5.0f;

    if(key == GLUT_KEY_LEFT)
        yRot -= 5.0f;

    if(key == GLUT_KEY_RIGHT)
        yRot += 5.0f;

    if(key > 356.0f)
        xRot = 0.0f;

    if(key < -1.0f)
        xRot = 355.0f;

    if(key > 356.0f)
        yRot = 0.0f;

    if(key < -1.0f)
        yRot = 355.0f;

    // Refresh the Window
    glutPostRedisplay();
    }


void ChangeSize(int w, int h)
    {
    GLfloat fAspect;

    // Prevent a divide by zero
    if(h == 0)
        h = 1;

  
    // Set Viewport to window dimensions
    glViewport(0, 0, w, h);

    // Reset coordinate system
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    fAspect = (GLfloat)w/(GLfloat)h;
    gluPerspective(60.0f, fAspect, 200.0, 5000.0);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    // Move out Z axis so we can see everything
    glTranslatef(0.0f, 0.0f, -350.0f);
    glLightfv(GL_LIGHT0,GL_POSITION,lightPos);
    }

void OnTimer(int )
{
	theta += 0.5f;
	if (theta > 360)
		theta = 0.0f;
	alpha += 0.6f;
	if (alpha > 360)
		alpha = 0.0f;
	dy += ddz;
	if ((dy > 10) || (dy < -10))
		ddz = -ddz;

	 // Redraw the scene
    glutPostRedisplay();
    
    // Reset timer
    glutTimerFunc(1,OnTimer, 1);
	
}

int main(int argc, char* argv[])
    {
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
    glutInitWindowSize(800, 600);
    glutCreateWindow("Shadow");
    glutReshapeFunc(ChangeSize);
    glutSpecialFunc(SpecialKeys);
    glutDisplayFunc(RenderScene);
	 glutTimerFunc(10, OnTimer, 1);
    SetupRC();
    glutMainLoop();

    return 0;
    }
